package com.edgec.browserbackend.browser.repository;

import com.edgec.browserbackend.browser.domain.IpResource;
import com.edgec.browserbackend.browser.domain.IpTransaction;
import com.mongodb.client.result.UpdateResult;
import org.bson.Document;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.aggregation.MatchOperation;
import org.springframework.data.mongodb.core.aggregation.SampleOperation;
import org.springframework.data.mongodb.core.query.BasicQuery;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Update;

import java.time.Instant;
import java.util.List;

import static org.springframework.data.mongodb.core.query.Criteria.where;

public class IpTransactionRepositoryCustomImpl implements IpTransactionRepositoryCustom {

    @Autowired
    MongoTemplate mongoTemplate;


    @Override
    public boolean lockTask(IpTransaction ipTransaction) {
        Document doc = new Document();
        BasicQuery basicQuery = new BasicQuery(doc);
        Criteria criteria = new Criteria();
        criteria.orOperator(where("tid").is(ipTransaction.getTid()).and("isLocked").is(false).and("status").is(ipTransaction.getStatus()),
                where("lockTimestamp").lte(Instant.now().minusSeconds(300).toEpochMilli()).and("status").is(ipTransaction.getStatus()));
        basicQuery.addCriteria(criteria);
        Update update = new Update();
        update.set("isLocked", true).set("lockTimestamp", Instant.now().toEpochMilli());
        UpdateResult result = mongoTemplate.updateFirst(basicQuery, update, IpResource.class);

        if (result.getModifiedCount() < 1)
            return false;
        else
            return true;
    }

    @Override
    public boolean unLockTask(String id) {
        Document doc = new Document();
        BasicQuery basicQuery = new BasicQuery(doc);
        basicQuery.addCriteria(where("id").is(id));
        Update update = new Update();
        update.set("isLocked", false).set("lockTimestamp", Instant.now().toEpochMilli());
        UpdateResult result = mongoTemplate.updateFirst(basicQuery, update, IpResource.class);

        if (result.getModifiedCount() < 1)
            return false;
        else
            return true;
    }

    @Override
    public List<IpTransaction> sampleTasks(int status, long timestamp) {
        Criteria matchCriteria  = new Criteria();
        matchCriteria.orOperator(where("status").is(status).and("isLocked").is(false),
                where("isLocked").is(true).and("lockTimestamp").lte(timestamp));

        MatchOperation match = Aggregation.match(matchCriteria);

        SampleOperation sample = Aggregation.sample(20);
        AggregationResults<IpTransaction> results = mongoTemplate.aggregate(Aggregation.newAggregation(match, sample), IpTransaction.class, IpTransaction.class);
        List<IpTransaction> mappedResults = results.getMappedResults();

        return mappedResults;
    }
}
